@@ -430,6 +430,20 @@ void getd2P(double *eva, double *ev, double *evi, int m, double el, double w, do
 }
 
 
+void getd2P2(double *eva, double *ev, double *evi, int m, double el, double w, double *result){
+	int i, j, h;
+	double tmp[m], res;
+	for(i = 0; i < m; i++) tmp[i] = (eva[i] * w) * (eva[i] * w) * exp(eva[i] * w * el);
+	for(i = 0; i < m; i++){	
+		for(j = 0; j < m; j++){
+			res = 0.0;	
+			for(h = 0; h < m; h++)	res += ev[i + h*m] * tmp[h] * evi[h + j*m];
+			result[i+j*m] = res;	
+		}
+	}
+}
+
+
 SEXP LogLik(SEXP dlist, SEXP P, SEXP nr, SEXP nc, SEXP node, SEXP edge, SEXP nTips, SEXP mNodes){
 	R_len_t i, n = length(node);
 	int nrx=INTEGER(nr)[0], ncx=INTEGER(nc)[0], nt=INTEGER(nTips)[0], mn=INTEGER(mNodes)[0];
@@ -647,6 +661,37 @@ SEXP getd2PM(SEXP eig, SEXP nc, SEXP el, SEXP w){
     return(RESULT);
 } 
 
+
+SEXP getd2PM2(SEXP eig, SEXP nc, SEXP el, SEXP w){
+	R_len_t i, j, nel, nw;
+	int m=INTEGER(nc)[0], l=0;
+	double *ws=REAL(w);
+	double *edgelen=REAL(el);
+	double *eva, *eve, *evei;
+	SEXP P, RESULT;
+	nel = length(el);
+	nw = length(w);
+	eva = REAL(VECTOR_ELT(eig, 0));
+	eve = REAL(VECTOR_ELT(eig, 1));
+	evei = REAL(VECTOR_ELT(eig, 2));
+	PROTECT(RESULT = allocVector(VECSXP, nel*nw));	
+	double *p;
+	if(!isNewList(eig)) error("dlist must be a list");	
+	for(j=0; j<nel; j++){
+		for(i=0; i<nw; i++){
+	        PROTECT(P = allocMatrix(REALSXP, m, m));
+	        p = REAL(P);
+            getd2P2(eva, eve, evei, m, edgelen[j], ws[i], p);
+            SET_VECTOR_ELT(RESULT, l, P);
+            UNPROTECT(1); //P
+            l++;
+        }
+    }
+    UNPROTECT(1); //RESULT
+    return(RESULT);
+} 
+
+
 //(dad  * (child %*% P)) 	
 SEXP getM3(SEXP dad, SEXP child, SEXP P, SEXP nr, SEXP nc){
 	R_len_t i, n=length(P);
@@ -668,6 +713,7 @@ SEXP getM3(SEXP dad, SEXP child, SEXP P, SEXP nr, SEXP nc){
 	}	
 
 
+
 SEXP FS(SEXP eig, SEXP nc, SEXP el, SEXP w, SEXP g, SEXP dad, SEXP child, SEXP ld, SEXP nr, SEXP basefreq, SEXP weight,SEXP f0, SEXP ll0, SEXP ff0)
 {
 	R_len_t j, n=length(dad);	
@@ -734,6 +780,4 @@ SEXP FS(SEXP eig, SEXP nc, SEXP el, SEXP w, SEXP g, SEXP dad, SEXP child, SEXP l
     return (RESULT);
 }	
 
-
-
 	
